@@ -110,8 +110,6 @@ AbTop::~AbTop()
 #ifdef SPION
   delete spy;
 #endif
-	
-  delete toolbar;
 }
 
 
@@ -1275,7 +1273,7 @@ void AbTop::optionShowSpy(bool on)
 
 void AbTop::help()
 {
-  kapp->invokeHTMLHelp("kabalone/index.html", "");
+  kapp->invokeHelp();
 }
 
 
