@@ -9,10 +9,14 @@
  * Josef Weidendorfer, 28.8.97
 */
 
-#include <qdatetm.h>
+#include <qdatetime.h>
 #include <qstrlist.h>
-#include <kapp.h>
+
+#include <kconfig.h>
+#include <krandomsequence.h>
+
 #include "Board.h"
+#include <stdio.h>
 
 // #define MYTRACE 1
 
@@ -23,6 +27,9 @@
 #endif
 
 
+static int ratedPositions, wonPositions, searchCalled, moveCount;
+static int normalCount, pushCount, outCount, cutoffCount;
+
 /*********************** Class MainCombination *************************/
 
 void MainCombination::clear(int d)
@@ -101,6 +108,7 @@ Board::Board()
 	clear();
 	breakOut = bUpdateSpy = false;
 	boardOk = true;
+	spyLevel = 0;
 	debug = 0;
 	realMaxDepth = 1;
 	setFieldValues();
@@ -108,7 +116,10 @@ Board::Board()
 
 void Board::setFieldValues()
 {
-  int i, j = 0, k = 57;
+  int i, j = 0, k = 59;
+
+  for(i=0;i<5;i++)
+    if (ringDiff[i]<1) ringDiff[i]=1;
 
   fieldValue[0] = ringValue[0];
   for(i=1;i<7;i++) 
@@ -183,6 +194,7 @@ void Board::begin(int startColor)
 	color = startColor;
 	color1Count = color2Count = 14;
 	boardOk = true;
+        random.setSeed(0); // Initialize random sequence
 }
 
 void Board::clear()
@@ -197,7 +209,7 @@ void Board::clear()
 }
 
 /* generate moves starting at field <startField> */
-inline void Board::generateFieldMoves(int startField, MoveList& list)
+void Board::generateFieldMoves(int startField, MoveList& list)
 {
 	int d, dir, c, actField, left, right;
 	int opponent = (color == color1) ? color2 : color1;
@@ -459,6 +471,8 @@ void Board::playMove(const Move& m)
 		CHECK( field[f + dir] == free );
 		field[f + dir] = color;
 		break;
+	default:
+	  break;
 	}
 
 	if (m.isOutMove()) {
@@ -606,6 +620,8 @@ void Board::takeBack()
 		CHECK( field[f + dir] == color );
 		field[f + dir] = free;
 		break;
+	default:
+	  break;
 	}
 
 	if (--storedLast < 0) storedLast = MvsStored-1;
@@ -734,6 +750,8 @@ void Board::showMove(const Move& mm, int step)
 	case Move::move1:       /* (c .) => (. c) */
 	  field[f + dir] = afterMove ? colorNew : free;
 	  break;
+	default:
+	  break;
 	}
 }
 
@@ -771,6 +789,8 @@ void Board::showStart(const Move& m, int step)
 	case Move::right2:
 	  dir = direction[m.direction];
 	  field[f + dir] = colorNew;
+	default:
+	  break;
 	}
 }
 
@@ -778,7 +798,7 @@ void Board::showStart(const Move& m, int step)
 /* calc value out of possible moves starting at field <startField>
  * parameter color shadows class member variable !
  */
-inline int Board::calcFieldValue(int startField, int color, int value)
+int Board::calcFieldValue(int startField, int color, int value)
 {
 	int d, dir, c, actField;
 	int opponent = (color == color1) ? color2 : color1;
@@ -863,6 +883,28 @@ inline int Board::calcFieldValue(int startField, int color, int value)
 }
 
 
+int Board::validState()
+{
+  int c1 = 0, c2 = 0;
+  int i,j;
+       
+  for(i=0;i<AllFields;i++) {              
+    j=field[i];
+    if (j == color1) c1++;
+    if (j == color2) c2++;
+  }
+
+  color1Count = c1;
+  color2Count = c2;
+
+  if (c1==0 && c2==0) 
+    return empty;
+  if (c1>8 && c1<15 && c2>8 && c2<15) 
+    return valid;
+  return invalid;
+}
+
+
 
 /* Calculate a rating for actual position
  * 
@@ -928,9 +970,9 @@ void Board::changeRating()
 
 	/* the outermost ring */
 	tmp=fieldValue[37];
-	for(i=37;i<61;i++)
+	for(i=37;i<60;i++)
 	  fieldValue[i] = fieldValue[i+1];
-	fieldValue[61] = tmp;
+	fieldValue[60] = tmp;
 }
 
 /*
@@ -956,9 +998,9 @@ void Board::showHist()
 
 void indent(int d)
 {
-	char tmp[]="                                  ";
+	char tmp[]="                                        ";
 	tmp[d*3] = 0;
-	printf("%s",tmp);
+	printf(">        %s",tmp);
 }
 
 /*
@@ -971,131 +1013,226 @@ int Board::search(int depth, int alpha, int beta)
 	MoveList list;
 	bool stop = false;
 
+	searchCalled++;
+
 	/* We make a depth search for the following move types... */
-	int maxType = (depth < maxDepth/2) ? Move::maxMoveType() :
-	              (depth < maxDepth)   ? Move::maxPushType() : 
-                                             Move::maxOutType();
+	int maxType = (depth < maxDepth-1)  ? Move::maxMoveType() :
+	              (depth < maxDepth)    ? Move::maxPushType() : 
+	                                      Move::maxOutType();
 
 	generateMoves(list);
 
 #ifdef MYTRACE
-	printf(">>>>>>>> Depth %d\n", depth);
+
+	moveCount += list.getLength();
+
+	/*
+	  if (spyLevel>1) {
+	  indent(depth);
+	  printf("%s (%6d .. %6d) MaxType %s\n", depth,
+	  (color==color1)?"O":"X", alpha,beta,
+		 (depth < maxDepth-1) ? "Moving" :
+		 (depth < maxDepth)? "Pushing" : "PushOUT" ); 
+	}
+	*/
 #endif
 	
 	/* check for a old best move in main combination */
 	if (inMainCombination) {
-		m = mc[depth];
-		if (!list.isElement(m, 0)) 
-		  m.type = Move::none;
-		if (m.type == Move::none)
-		  inMainCombination = false;
-		if (m.type > maxType)
-		  m.type = Move::none;
+	  m = mc[depth];
+	  if (!list.isElement(m, 0, true)) 
+	    m.type = Move::none;
+	  if (m.type == Move::none)
+	    inMainCombination = false;
+#ifdef MYTRACE
+	  else {
+	    if (spyLevel>1) {		 
+	      indent(depth);
+	      printf("Got from MC !\n" );
+	    }
+	  }
+#endif
+	  /* for moves in MainCombination we do depth search
+	   * even if we don't normally in this depth => Comment out
+	   */
+	  //if (m.type > maxType)
+	  //  m.type = Move::none;
 	}
 	if (m.type == Move::none)
 	  stop = !list.getNext(m, maxType);
 
 	/* depth first search */
-	while(!stop) {
+	while(!stop) {	  
 
 #ifdef MYTRACE
-	  indent(depth);
-	  m.print();
-	  printf("\n");
+
+	  if (m.isOutMove()) outCount++;
+	  else if (m.isPushMove()) pushCount++;
+	  else normalCount++;
+
+	  int oldRatedPositions = ratedPositions;
+	  int oldWonPositions = wonPositions;
+	  int oldSearchCalled = searchCalled;
+	  int oldMoveCount = moveCount;
+	  int oldNormalCount = normalCount;
+	  int oldPushCount = pushCount;
+	  int oldOutCount = outCount;
+	  int oldCutoffCount = cutoffCount;
+	  
+	  if (spyLevel>1) {
+	    indent(depth);
+	    printf("%s [%6d .. %6d] ",
+		   (color==color1)?"O":"X", alpha,beta);
+	    m.print();
+	    printf("\n");
+	  }
 #endif	
 		
 #ifdef SPION
-	  	if (bUpdateSpy) emit update(depth, 0, m, false);
+	  if (bUpdateSpy) emit update(depth, 0, m, false);
 #endif
-		playMove(m);
-		if (!isValid())
-		  value = ((depth < maxDepth) ? 15999:14999) - depth;
-		else {
-		  /* opponent searches for his maximum; but we won't the
-		   * minimum: so change sign (for alpha/beta window too!)
-		   */
-		  value = - search(depth+1,-beta,-alpha);
-		}
-		takeBack();
-
-		/* For GUI response */
-		if (maxDepth - depth >2)
-		  emit searchBreak();		
-
-#ifdef MYTRACE	
-		indent(depth);
-		printf("=> (%d - %d): Value %d [ %d ] for ",
-		       alpha, beta, value,actValue);
-		m.print();
-	  printf("\n");
-#endif
-
-#ifdef SPION
-		if (bUpdateSpy) {
-		  if (value > actValue)
-		    emit updateBest(depth, value, m, value >= beta);
-		  emit update(depth, value, m, true);
-		}		  
-#endif
-		if (value > actValue) {
-			actValue = value;
-			mc.update(depth, m);
-
-			if (bUpdateSpy && depth == 0)
-			  emit updateBestMove(m, actValue);
-			
-			if (actValue >= beta) {
+	  playMove(m);
+	  if (!isValid()) {
+	    value = 14999-depth;
+	    //  value = ((depth < maxDepth) ? 15999:14999) - depth;
 #ifdef MYTRACE
-			  indent(depth);
-			  printf("CUTOFF\n");
+	    wonPositions++;
 #endif
-			  return actValue;
-			}			
-			if (actValue > alpha) alpha = actValue;
+	  }
+	  else {
+	    
+	    /* opponent searches for his maximum; but we won't the
+	     * minimum: so change sign (for alpha/beta window too!)
+	     */		  
+	    value = - search(depth+1,-beta,-alpha);
+	  }
+	  takeBack();
+	  
+	  /* For GUI response */
+	  if (maxDepth - depth >2)
+	    emit searchBreak();		
+	  
+#ifdef MYTRACE	
+	  if (spyLevel>1) {
+
+	    indent(depth);
+	    if (oldSearchCalled < searchCalled) {
+	      printf("  %d Calls", searchCalled-oldSearchCalled);
+	      if (cutoffCount>oldCutoffCount)
+		printf(" (%d Cutoffs)", cutoffCount-oldCutoffCount);
+	      printf(", GenMoves %d (%d/%d/%d played)",
+		     moveCount - oldMoveCount,
+		     normalCount - oldNormalCount,
+		     pushCount-oldPushCount,
+		     outCount-oldOutCount);
+	      printf(", Rate# %d",
+		     ratedPositions+wonPositions
+		     - oldRatedPositions - oldWonPositions);
+	      if (wonPositions > oldWonPositions)
+		printf(" (%d Won)", wonPositions- oldWonPositions);
+	      printf("\n");
+	      indent(depth);
+	    }
+	    
+	    printf("  => Rated %d%s\n",
+		   value,
+		   (value>14900)  ? ", WON !":
+		   (value>=beta)  ? ", CUTOFF !":
+		   (value>actValue)  ? ", Best !": "");
+	  }
+
+	  if (value>=beta) cutoffCount++;
+#endif
+	  
+#ifdef SPION
+	  if (bUpdateSpy) {
+	    if (value > actValue)
+	      emit updateBest(depth, value, m, value >= beta);
+	    emit update(depth, value, m, true);
+	  }		  
+#endif
+	  if (value > actValue) {
+	    actValue = value;
+	    mc.update(depth, m);
+	    
+	    if (bUpdateSpy && depth == 0) {
+	      emit updateBestMove(m, actValue);
+#ifdef MYTRACE	
+	      if (spyLevel>0) {
+		int i;
+		printf(">      New MC (Rating %d):", actValue);
+		for(i=0;i<=maxDepth;i++) {
+		  printf("\n>          D %d: %s", 
+			 i, (const char*)(mc[i].name()) );
 		}
-		
-		stop = (!list.getNext(m, maxType)) || breakOut;
+		printf("\n>\n");
+	      }
+#endif
+	    }
+
+	    if (actValue>14900 || actValue >= beta)
+	      return actValue;
+	    
+	    /* maximize alpha */
+	    if (actValue > alpha) alpha = actValue;
+	  }
+	  
+	  stop = (!list.getNext(m, maxType)) || breakOut;
 	}
 	
 	/* other moves: calculate rating */
 	while(list.getNext(m, Move::none)) {
+
+	  if (m.isOutMove()) outCount++;
+	  else if (m.isPushMove()) pushCount++;
+	  else normalCount++;
 		
-		playMove(m);
-		if (!isValid())
-		  value = ((depth < maxDepth) ? 15999:14999) - depth;
-		else
-		  value = calcValue();
-		takeBack();
+	  playMove(m);
+	  if (!isValid()) {
+	    value = 14999 - depth;
+	    //  value = ((depth < maxDepth) ? 15999:14999) - depth;
+#ifdef MYTRACE
+	    wonPositions++;
+#endif
+	  }
+	  else {
+	    ratedPositions++;
+	    
+	    value = calcValue();
+	  }
+	  takeBack();
 
 #ifdef SPION
-		if (bUpdateSpy) {
-		  if (value > actValue)
-		    emit updateBest(depth, value, m, value >= beta);
-		  emit update(depth, value, m, true);
-		}		  
+	  if (bUpdateSpy) {
+	    if (value > actValue)
+	      emit updateBest(depth, value, m, value >= beta);
+	    emit update(depth, value, m, true);
+	  }		  
 #endif
 
 #ifdef MYTRACE
-		indent(depth);
-		printf("> (%d - %d): Value %d [ %d ] for ",
-		       alpha, beta, value, actValue);
-		m.print();
-		printf("\n");
-#endif
-		if (value > actValue) {
-			actValue = value;
-			mc.update(depth, m);
+	  if (spyLevel>2) {
+	    indent(depth);
+	    printf("%s (%6d .. %6d) %-25s => Rating %6d%s\n",
+		   (color==color1)?"O":"X", alpha,beta,
+		   (const char*)m.name(),
+		   value,
+		   (value>14900)  ? ", WON !":
+		   (value>=beta)  ? ", CUTOFF !":
+		   (value>actValue)  ? ", Best !": "");
+	  }
 
-			if (actValue >= beta) {
-#ifdef MYTRACE
-			  indent(depth);
-			  printf("CUTOFF\n");
+	  if (value>=beta) cutoffCount++;
 #endif
-			  break;
-			}
+	  if (value > actValue) {
+	    actValue = value;
+	    mc.update(depth, m);
+	    
+	    if (actValue>14900 || actValue >= beta)
+	      return actValue;
 			
-			if (actValue > alpha) alpha = actValue;
-		}
+	    if (actValue > alpha) alpha = actValue;
+	  }
 	}	
 	return actValue;
 }
@@ -1112,62 +1249,108 @@ Move& Board::bestMove()
 	show = false;
 	breakOut = false;
 
-#ifdef MYTRACE
-	printf(">>>>>>>>>>>>>>>>>> New Search\n");
-#endif
+	if (spyLevel>0)
+	  printf("\n> New Search\n>\n");
+
 	do {
-#ifdef MYTRACE
-		printf(">> MaxDepth: %d\n", maxDepth);
-#endif
+	  if (spyLevel>0)
+	    printf(">   MaxDepth: %d\n>\n", maxDepth);
+	  
 		// ShowTiefe(maxtiefe);
 		do {
-#ifdef MYTRACE
-			  printf(">> Alpha/Beta: (%d ... %d)\n", alpha, beta);
-#endif
-			nalpha=alpha, nbeta=beta;
-			inMainCombination = (mc[0].type != Move::none);
-			actValue = search(0,alpha,beta);
-			
-			/* Don't break out if we haven't found a move */
-			if (mc[0].type == Move::none) 
-			  breakOut=false;
-			if (actValue > 15000 || actValue < -15000)
-			  breakOut=true;
-			if (actValue <= nalpha)
-			  alpha = -15000, beta=actValue+1;
-			if (actValue >= nbeta)
-			  alpha = actValue-1, beta=15000;
+		  if (spyLevel>0)
+		    printf(">     AB-Window: (%d ... %d)\n>\n", alpha, beta);
+
+		  nalpha=alpha, nbeta=beta;
+		  inMainCombination = (mc[0].type != Move::none);
+
+		  /* Statistics */
+		  searchCalled = 0;
+		  moveCount = 0;
+		  ratedPositions = 0;
+		  wonPositions = 0;
+		  normalCount = 0;
+		  pushCount = 0;
+		  outCount = 0;
+		  cutoffCount = 0;
+		  
+		  actValue = search(0,alpha,beta);
+		  
+		  if (spyLevel>0)
+		    {
+		      int i;
+		      if (spyLevel>1)
+			printf(">\n");
+		      printf(">      Got MC with Rating %d:",actValue);
+		      for(i=0;i<=maxDepth;i++) {
+			printf("\n>          D %d: ", i);
+			mc[i].print();
+		      }
+		      printf("\n>\n");
+
+		      printf(">      Search called    : %6d / %d Cutoffs\n",
+			     searchCalled, cutoffCount);
+		      printf(">       Moves generated : %6d / %d Played\n",
+			     moveCount, normalCount+pushCount+outCount);
+		      printf(">        Nrml/Push/Out  : %6d / %d / %d\n",
+			     normalCount,pushCount,outCount);
+		      printf(">       Positions rated : %6d / %d Won\n>\n",
+			     ratedPositions+wonPositions, wonPositions);
+
+		    }
+
+		  /* Don't break out if we haven't found a move */
+		  if (mc[0].type == Move::none) 
+		    breakOut=false;
+		  if (actValue > 14900 || actValue < -14900)
+		    breakOut=true;
+		  if (actValue <= nalpha)
+		    alpha = -15000, beta=actValue+1;
+		  if (actValue >= nbeta)
+		    alpha = actValue-1, beta=15000;
 		}
 		while(!breakOut && (actValue<=nalpha || actValue>=nbeta));
-		//		if ( (maxDepth+((color == color2)?1:0)) %2 ==1)
-		alpha=actValue-200, beta=actValue+1;
-		// else
-		// alpha=actValue-1, beta=actValue+200;
+
+		/* Window in both directions cause of deepening */
+		alpha=actValue-200, beta=actValue+200;
+		/*
+		  if ( (maxDepth+((color == color2)?1:0)) %2 ==1)
+		  alpha=actValue-200, beta=actValue+1;
+		  else
+		  alpha=actValue-1, beta=actValue+200;
+		*/
 		// ShowHv2();
+		/*
+		  if (spyLevel>0) {
+		  printf(">>> Best Move so far: %d - ", actValue);
+		  mc[0].print();
+		  printf("\n");
+		  }	
+		*/	
 	}	
 	while(++maxDepth< realMaxDepth && !breakOut);
-	
-	changeRating();
-	
+
+	/* If Spy is On, we want replayable search: don't change rating! */
+	if (spyLevel==0)
+	  changeRating();
+	else {
+	  printf(">>> Got Move : ");
+	  mc[0].print();
+	  printf("\n\n");
+	}
+
 	return(mc[0]);
 }	  
 
 Move Board::randomMove()
 {
-	static int i = 999;
-	unsigned int j,l;
-
 	Move m;
 	MoveList list;
 	
-	/* we prefer to use QT... */
-	j = (QTime::currentTime()).msec();
-	
 	generateMoves(list);
-	l = list.getLength();
+	int l = list.getLength();
 
-	j = (j + i) % l +1;
-	if ( (i+=7)>10000) i-=10000;
+	int j = random.getLong(l) +1;
 	
 	while(j != 0) {
 		list.getNext(m, Move::none);
@@ -1178,11 +1361,11 @@ Move Board::randomMove()
 }
 	
 
-void Board::print()
+void Board::print(int )
 {
 	int row,i;
 	char spaces[]="      ";
-	char *z[]={". ","O ","X ", "o ", "x "};
+	const char *z[]={". ","O ","X ", "o ", "x "};
 
 	printf("\n       -----------\n");
 	for(row=0;row<4;row++) {
@@ -1202,46 +1385,150 @@ void Board::print()
 	       color1Count, color2Count);	
 }
 
-QString Board::getState()
+QString Board::getASCIIState(int moveNo)
 {
-	QString state;
-	QString entry, tmp;
-	Move m;
+  QString state, tmp;
 
-	/* remove any highlighting */
-	showMove(m,0);
+  int row,i;
+  char spaces[]="      ";
+  const char *z[]={". ","O ","X ", "o ", "x "};
 
-	/* Color + Counts */
-	state += (char) ('A' + color1Count);
-	state += (char) ('A' + color2Count);
-	state += (char) ('A' + 3*color + field[order[0]]);
-	
-	/* Board (field values can be 0/1/2; 3 fields coded in one char */
-	for(int i=1;i<=60;i+=3)
-	  state+= (char) ('@' + 9*field[order[i]] + 
-			  3*field[order[i+1]] + field[order[i+2]] );
+  state += tmp.sprintf("\n #%-3d  -----------     O: %d  X: %d\n",
+		       moveNo, color1Count, color2Count);  
+  for(row=0;row<4;row++) {
+    state += tmp.sprintf("%s/ ",spaces+row);    
+    for(i=0;i<5+row;i++) 
+      state += tmp.sprintf("%s",z[field[row*11+12+i]]);
+    state += "\\\n";
+  }
+  state += "  | ";
+  for(i=0;i<9;i++)
+    state += tmp.sprintf("%s",z[field[56+i]]);
+  state += "|\n";
+  for(row=0;row<4;row++) {
+    state += tmp.sprintf("%s\\ ",spaces+3-row);
+    for(i=0;i<8-row;i++) 
+      state += tmp.sprintf("%s",z[field[68+row*12+i]]);
+    state += "/\n";
+  }
+  state += "       -----------\n\n";
 
-	/* -> 23 chars */
-	return state;
-}	
-
-void Board::setState(QString& state)
-{
-	if (state.length() != 23) return;
-
-	color1Count = state[0] - 'A';
-	color2Count = state[1] - 'A';
-	color = (state[2] - 'A') / 3;
-	field[order[0]] = (state[2] - 'A') %3;
-
-	int i = 1;
-	for(int j = 3; j<23; j++) {
-		int w = state[j] - '@';
-		field[order[i++]] = w/9;
-		field[order[i++]] = (w % 9)/3;
-		field[order[i++]] = w % 3;
-	}
+  return state;
 }
 
-#include "Board.moc"
+int Board::setASCIIState(const QString& state)
+{
+  int moveNo=-1, index;
+  int len = state.length();
+  int color1Count = 0;
+  int color2Count = 0;
 
+  /* get moveNo if supplied */
+  if ((index = state.find("#"))>=0)
+    moveNo = state.mid(index+1,3).toInt();
+
+  int f=12, row=0, rowEnd = 17;
+  char c;
+
+  index=state.find("/");
+
+  while(index>=0) {
+
+    while(++index<len) {
+      c = state[index].latin1();
+      if (c !=' ') break;
+    }
+
+    if (c=='.') field[f++] = 0;
+    else if (c=='o' || c=='O') {
+      //      printf(".. F %d: O\n", f);
+      field[f++] = 1;
+      color1Count++;
+    }
+    else if (c=='x' || c=='X') {
+      //      printf(".. F %d: X\n", f);
+      field[f++] = 2;
+      color2Count++;
+    }
+    else {
+      if (index >= len) index=-1;
+      continue;
+    }
+
+    if (f == rowEnd) {
+      row++;
+      if (row <4) {
+	index = state.find("/",index);
+	f = 12 + row*11;
+	rowEnd = row*12+17;
+      }
+      else if (row==4) {
+	index = state.find("|",index);
+	f = 56;
+	rowEnd = 65;
+      }
+      else if (row <9) {
+	index = state.find("\\",index);
+	f = 8 + row*12;
+	rowEnd = 21 + row*11;
+      }
+      else
+	break;
+      //      printf("Row %d: %d - %d, Idx %d\n", row, f, rowEnd, index);
+    }
+  }
+  return moveNo;
+}
+
+
+QString Board::getState(int moveNo)
+{
+  QString state;
+  QString entry, tmp;
+  Move m;
+  int i;
+  
+  /* remove any highlighting */
+  showMove(m,0);
+  
+  /* Color + Counts */
+  state += (char) ('A' + moveNo /25 );
+  state += (char) ('A' + moveNo %25 );
+  state += (char) ('A' + color1Count);
+  state += (char) ('A' + color2Count);
+  state += (char) ('A' + 4*color + field[order[0]]);
+  
+  /* Board (field values can be 0-3; 2 fields coded in one char */
+  for(i=1;i<61;i+=2)
+    state+= (char) ('A' + 4*field[order[i]] + field[order[i+1]] );
+  
+  /* -> 35 chars */
+  return state;
+}
+
+int Board::setState(QString& _state)
+{
+  int moveNo;
+  const char *state = _state.ascii();
+
+  if (_state.length() != 35) return 0;
+
+  moveNo = 25*(state[0] - 'A') + (state[1] - 'A');
+  color1Count = state[2] - 'A';
+  color2Count = state[3] - 'A';
+  color = (state[4] - 'A') / 4;
+  field[order[0]] = (state[4] - 'A') %4;
+  
+  int i = 1;
+  for(int j = 5; j<35; j++) {
+    int w = state[j] - 'A';
+    field[order[i++]] = w/4;
+    field[order[i++]] = w % 4;
+  }
+  return moveNo;
+}
+
+void Board::setSpyLevel(int level)
+{
+  spyLevel = level;  
+}
