@@ -35,9 +35,9 @@ int main( int argc, char *argv[] )
     KApplication app;
     KGlobal::dirs()->addResourceType("sprite", KStandardDirs::kde_default("data") + "kasteroids/sprites/");
     
-    KAstTopLevel mainWidget;
-    mainWidget.show();
-    app.setMainWidget( &mainWidget );
+    KAstTopLevel *mainWidget = new KAstTopLevel();
+    app.setMainWidget( mainWidget );
+    mainWidget->show();
 
     app.exec();
 
