@@ -5,14 +5,18 @@
 #include <kstatusbar.h>
 #include <ktoolbar.h>
 #include <ktmainwindow.h>
-#include <qlcdnumber.h>
-#include <qdict.h>
 #include <kmenubar.h>
+#include <kaccel.h>
+#include <qdict.h>
+#include <qmap.h>
 
-#include "kfixedtopwidget.h"
 #include "view.h"
+#include "highscore.h"
 
-class KAstTopLevel : public KFixedTopWidget
+class KALedMeter;
+class QLCDNumber;
+
+class KAstTopLevel : public KTMainWindow
 {
     Q_OBJECT
 public:
@@ -21,9 +25,9 @@ public:
 
 private:
     void createMenuBar();
-    void readSettings();
     void playSound( const char *snd );
     void readSoundMapping();
+    void doStats();
 
 protected:
     virtual void keyPressEvent( QKeyEvent *event );
@@ -32,6 +36,7 @@ protected:
     virtual void focusOutEvent( QFocusEvent *event );
 
 private slots:
+    void readSettings(); // it is useful as a slot
     void slotNewGame();
     void slotQuit();
 
@@ -39,6 +44,13 @@ private slots:
     void slotRockHit( int size );
     void slotRocksRemoved();
 
+    void slotUpdateVitals();
+
+    void slotKeyConfig();
+    void slotPref();
+    void slotShowHighscores();
+    void slotReturn();  // returning from dialogs - unpausing the game really...
+
 private:
     KAsteroidsView *view;
     KMenuBar *menu;
@@ -46,15 +58,31 @@ private:
     QLCDNumber *levelLCD;
     QLCDNumber *shipsLCD;
 
+    QLCDNumber *teleportsLCD;
+//    QLCDNumber *bombsLCD;
+    QLCDNumber *brakesLCD;
+    QLCDNumber *shieldLCD;
+    KALedMeter *powerMeter;
+
+    HighscoreList *highscore;
+
     bool   sound;
     QDict<QString> soundDict;
 
     // waiting for user to press Enter to launch a ship
     bool waitShip;
+    bool isPaused;
 
     int shipsRemain;
     int score;
     int level;
+    bool showStats, showHiscores;
+
+    enum Action { Launch, Thrust, RotateLeft, RotateRight, Shoot, Teleport,
+                    Brake, Shield, Pause };
+
+    KAccel *accel;
+    QMap<QString,Action> actions;
 };
 
 #endif
