@@ -14,11 +14,13 @@
 
 #include "kbbgfx.h"
 #include "util.h"
+
 #include <qwidget.h>
 #include <qmenubar.h> 
 #include <qlabel.h>
-#include <qpushbt.h>
-#include <ktopwidget.h>
+#include <qpushbutton.h>
+#include <ktmainwindow.h>
+#include <krandomsequence.h>
 
 /*
    Types of the boxes (used f.e.g. in the traceRay() method)
@@ -42,8 +44,6 @@
  * Menu & toolbar IDs.
  */
 #define ID_HELP     0
-#define ID_ABOUT    1
-#define ID_ABOUT_QT 2
 #define ID_QUIT     3
 #define ID_NEW      4
 #define ID_GIVEUP   5
@@ -58,7 +58,7 @@
 #define SRUN   2
 #define SSIZE  3
 
-class KBBGame : public KTopLevelWidget
+class KBBGame : public KTMainWindow
 {
   Q_OBJECT
 public:
@@ -83,12 +83,10 @@ private slots:
   void balls3();
 
   void tutorialSwitch();
+
+  void help();
   
   void newGame();
-  void about();
-//  void aboutKDE();
-  void aboutQt();
-  void help();
 
   bool setSize( int w, int h );
   bool setBalls( int n );
@@ -126,6 +124,7 @@ private:
   int score;
   /*  QLabel *scoreText;
   QLabel *statusText;*/
+  KRandomSequence random;
 };
 
 #endif // KBBGAME_H
