@@ -20,7 +20,6 @@
 
 #include <qdialog.h>
 #include "freecell.h"
-#include <kmsgbox.h>
 #include "global.h"
 
 #define STACK 1
@@ -69,8 +68,6 @@ void Freecell::show()
 Freecell::Freecell( QWidget* parent, const char* name)
 	: dealer(parent,name)
 {
-	initMetaObject();
-
 	freecell_game = this;
 
 	deck = new Deck (-666, -666, this);
