@@ -12,16 +12,7 @@
  *     You should have received a copy of the GNU General Public License
  *     along with this program; if not, write to the Free Software
  *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- *
- *
- *
- * Comment:
- * This is my first "bigger" application I did with Qt and my very first KDE application.
- * So please apologize some strange coding styles --> if you can't stand some really
- * bad parts just send me a patch including you "elegant" code ;)
- *
- * WABA: Who are you?
-*/
+ */
     
 #include <kapp.h>
 #include <klocale.h>
@@ -39,10 +30,13 @@ KConfig *conf;
 
 int main( int argc, char *argv[] )
 {
-	KAboutData aboutData( "kpoker", I18N_NOOP("KPoker"), 
+	KAboutData aboutData("kpoker", I18N_NOOP("KPoker"), 
 		KPOKER_VERSION, description, KAboutData::License_GPL, 
-		"(c) 1997-1998, Jochen Tuchbreiter");
-	aboutData.addAuthor("Jochen Tuchbreiter",0, "whynot@mabi.de");
+		"(c) 1997-2000 Jochen Tuchbreiter, Andreas Beckermann", 
+		I18N_NOOP("For a full list of credits see helpfile\nAny suggestions, bug reports etc. are welcome"));
+
+	aboutData.addAuthor("Andreas Beckermann", I18N_NOOP("Current maintainer"), "b_mann@gmx.de");
+	aboutData.addAuthor("Jochen Tuchbreiter", I18N_NOOP("Original author"), "whynot@mabi.de");
 	KCmdLineArgs::init( argc, argv, &aboutData );
 
 	KApplication myApp;
@@ -58,8 +52,6 @@ int main( int argc, char *argv[] )
 		  
 	int ret = myApp.exec();
 		  
-	delete MainScreen;
-	
 	return ret;
 
 }
