@@ -21,40 +21,38 @@
 
 #ifndef _HighScoreWIDGET
 #define _HighScoreWIDGET
-#include <qwindow.h>
-#include <qpushbt.h>
 
-class HighScore : public QWindow {
- Q_OBJECT
-public:
- HighScore(QWidget *parent=0);     
-     void add(int board, int score,int colors);
-     
-protected:
-     void add(int board, int score,int colors,char *name);
-     void load();
-     void store();
+#include <qdialog.h>
+#include <qstring.h>
 
-private:
 #define HS_MAXENTRY 10
+
+class HighScore {
+
+public:
+     HighScore(); 
+     ~HighScore();
+     void add(int board, int score,int colors);
+     void showScore(bool forceNew=false);     
+protected:
+     void add(int board, int score,int colors,const QString &name);
+     void loadScore();
+     void storeScore();
+     
+private:
      struct HSEntry {
 	  int board;
 	  int score;
 	  int colors;
-	  char name[17];
+	  QString name;
      };
 
      HSEntry hiscore[HS_MAXENTRY];
      int hiscore_used;
      
-     char playername[17];
-
-     int fontw,fonth;
-     QSize mSizeHint;
-     QPushButton *ok;
-     void keyPressEvent (QKeyEvent * e);
-     virtual void paintEvent ( QPaintEvent * ); 
-     virtual QSize sizeHint() { return mSizeHint; }
+     QString playername;
+     
+     QDialog *hs_dlg;
 };
 
 #endif
