@@ -1,3 +1,4 @@
+/* Yo Emacs, this is -*- C++ -*- */
 /*
  *   ksame 0.4 - simple Game
  *   Copyright (C) 1997,1998  Marcus Kreutzberger
@@ -20,62 +21,54 @@
 
 #ifndef _KSAMEWIDGET
 #define _KSAMEWIDGET
-#include <qwidget.h>
-#include <qpushbt.h>
-#include <ktopwidget.h> 
-#include <qlist.h>
-#include <qpoint.h>
+
 #include "StoneWidget.h"
 #include "HighScore.h"
+#include <ktmainwindow.h>
 
-class KSameWidget: public KTopLevelWidget {
- Q_OBJECT
+class KSameWidget: public KTMainWindow {
+     Q_OBJECT
 public:
-    KSameWidget(); 
-    ~KSameWidget();
- 
+     KSameWidget(); 
+     ~KSameWidget();
+     
 private:
-    StoneWidget *stone;
-    HighScore *highscore;
-    KStatusBar *status;
-    
-    QList<QPoint> history;
+     StoneWidget *stone;
+     HighScore *highscore;
+     KStatusBar *status;
+     
+     QPopupMenu *colorsmenu;
+     QPopupMenu *optionsmenu;
+     QPopupMenu *editmenu;
+     int multispin_item;
+     int random_item;
+     
+     int israndom;
 
-    QPopupMenu *colorsmenu;
-    QPopupMenu *optionsmenu;
-    int multispin_item;
-    int random_item;
 
-    int israndom;
-    int board;
-
-    void set_board(int no);
 protected:
-    void load_history();
-    void store_history();
-
-    virtual void saveProperties(KConfig *conf);
-    virtual void readProperties(KConfig *conf);
-public slots: 
-    void sizeChanged();   
-  /* File Menu */
-    void m_new();
-    void m_load();
-    void m_save();
-    void m_showhs();
-    void m_quit();
+     void newGame(unsigned int board,int colors);
+     
+     virtual void saveProperties(KConfig *conf);
+     virtual void readProperties(KConfig *conf);
+  public slots: 
+  void sizeChanged();   
+     /* File Menu */
+     void m_new();
+     void m_load();
+     void m_save();
+     void m_showhs();
+     void m_quit();
+     void m_undo();
+     
+     /* Options Menu */
+     void m_tglboard();
     
-    
-
-    /* Options Menu */
-    void m_colors(int);
-    void m_tglboard();
-    
-    void gameover();
-    void set_score(int);
-    void set_marked(int); 
-    void history_add(int,int);
-    void history_clear();
+     void gameover();
+     void setColors(int colors);
+     void setBoard(int board);
+     void setScore(int score);
+     void setMarked(int m);      
 };
 
 
