@@ -37,17 +37,22 @@
  */
 
 #include "board.h"
-#include <stdlib.h>
 #include <stdio.h>
+#include <stdlib.h>
 #include <qpainter.h>
 #include <kapp.h>
 #include <qimage.h>
 #include <qtimer.h>
 #include <qlist.h>
 #include <sys/times.h>
-#include <qbitarry.h>
-#include <kmsgbox.h>
+#include <qbitarray.h>
 #include <debug.h>
+#include <klocale.h>
+#include <kiconloader.h>
+#include <kglobal.h>
+#include <kstddirs.h>
+#include <qbitmap.h>
+#include <qmessagebox.h>
 
 #ifdef DEBUGGING
 #include <unistd.h>
@@ -59,31 +64,31 @@
 #define DEFAULTDELAY	500
 #define DEFAULTSHUFFLE	4
 
-extern QString PICDIR;
-
 Board::Board(QWidget *parent) : QWidget(parent) {
-  trying = FALSE;
-  _solvable_flag = TRUE;
+  pausedIcon = 0;
+  paused = false;
+  trying = false;
+  _solvable_flag = true;
+  grav_col_1 = -1;
+  grav_col_2 = -1;
+  setGravityFlag(false);
 
   // randomze
   setShuffle(DEFAULTSHUFFLE);
-  clock_t t;
-  struct tms dummy;
-  t = times(&dummy); 
-  srandom((int)t);
 
+  random.setSeed(0);
   starttime = time((time_t *)0);
 
   for(int i = 0; i < 45; i++)
     pm_tile[i] = 0;
 
   setDelay(DEFAULTDELAY);
-  _redo.setAutoDelete(TRUE);
-  _undo.setAutoDelete(TRUE);
+  _redo.setAutoDelete(true);
+  _undo.setAutoDelete(true);
 
   loadTiles(1);
   field = 0;
-  QPixmap bg((PICDIR + "/kshisen_bgnd.xpm").data());
+  QPixmap bg(KGlobal::dirs()->findResource("appdata", "kshisen_bgnd.xpm"));
   setBackgroundPixmap(bg);
   connect(this, SIGNAL(fieldClicked(int, int)),
 	  this, SLOT(marked(int, int)));
@@ -133,6 +138,30 @@ int Board::getField(int x, int y) {
     return field[y * x_tiles() + x];
 }
 
+void Board::gravity(int col, bool update) {
+  if(gravity_flag) {
+    int rptr = y_tiles()-1, wptr = y_tiles()-1;
+    while(rptr >= 0) {
+      if(getField(col, wptr) != EMPTY) {
+	rptr--;
+	wptr--;
+      } else {
+	if(getField(col, rptr) != EMPTY) {
+	  setField(col, wptr, getField(col, rptr));
+	  setField(col, rptr, EMPTY);
+	  if(update) {
+	    updateField(col, rptr);
+	    updateField(col, wptr);
+	  }
+	  wptr--;
+	  rptr--;
+	} else
+	  rptr--;	
+      }
+    }
+  }
+}
+
 void Board::mousePressEvent(QMouseEvent *e) {
     // calculate position
     int pos_x = (e->pos().x() - XBORDER <0)?-1:
@@ -185,9 +214,9 @@ void Board::setSize(int x, int y) {
     return;
 
   if(field != 0)
-    free(field);
+    delete [] field;
 
-  field = (int*)malloc(sizeof(int) * x * y);
+  field = new int[ x * y ];
   _x_tiles = x;
   _y_tiles = y;
   for(int i = 0; i < x; i++)
@@ -217,22 +246,29 @@ bool Board::loadTiles(float scale) {
       pm_tile[i] = 0;
     }
 
-  QPixmap pm((PICDIR + "/kshisen.xpm").data());
+  // locate tileset
+  QPixmap pm(KGlobal::dirs()->findResource("appdata", "kshisen.xpm"));
+  QBitmap mask(KGlobal::dirs()->findResource("appdata", "mask.xpm"));
   if(pm.width() == 0 || pm.height() == 0) {
-    KMsgBox::message(0, klocale->translate("Shisen-Sho"), 
-		     klocale->translate("Cannot load pixmaps!"));
-    exit(1);
+      QMessageBox::warning(this, kapp->caption(),
+                           i18n("Cannot load pixmaps!"), 
+                           i18n("OK"));
+      exit(1);
   }
+  pm.setMask(mask);
   
   if(pm.width() == 0 || pm.height() == 0)
-    return FALSE;
+    return false;
 
   x = pm.width() / 9;
   y = pm.height() / 5;
   for(i = 0; i < 9; i++)
     for(j = 0; j < 5; j++) {
 	pm_tile[i + j*9] = new QPixmap(x,y);
+	QBitmap bm(x, y);
 	bitBlt(pm_tile[i + j*9], 0, 0, &pm, x * i, y * j, x, y, CopyROP);
+	bitBlt(&bm, 0, 0, &mask, x *i, y * j, x, y, CopyROP);
+	pm_tile[i+j*9]->setMask(bm);
 	if(scale != 1.0) {
 	  QWMatrix wm;
 	  wm.scale(scale, scale);
@@ -240,7 +276,7 @@ bool Board::loadTiles(float scale) {
 	}
     }
   
-  return TRUE;
+  return true;
 }
 
 void Board::newGame() {
@@ -300,16 +336,16 @@ void Board::newGame() {
   int tx = x_tiles();
   int ty = y_tiles();
   for(i = 0; i < x_tiles() * y_tiles() * getShuffle(); i++) {
-    int x1 = random(tx);
-    int y1 = random(ty);
-    int x2 = random(tx);
-    int y2 = random(ty);
+    int x1 = random.getLong(tx);
+    int y1 = random.getLong(ty);
+    int x2 = random.getLong(tx);
+    int y2 = random.getLong(ty);
     int t  = getField(x1, y1);
     setField(x1, y1, getField(x2, y2));
     setField(x2, y2, t);
   }
 
-  // do not make solvable if _solvable_flag is FALSE
+  // do not make solvable if _solvable_flag is false
   if(!_solvable_flag) {
     if(!trying) {
       update();
@@ -326,7 +362,7 @@ void Board::newGame() {
   int *tiles = new int[x_tiles() * y_tiles()];
   int *pos = new int[x_tiles() * y_tiles()];
 
-  while(!solvable(TRUE)) {
+  while(!solvable(true)) {
     // generate a list of free tiles and positions
     int num_tiles = 0;
     for(i = 0; i < x_tiles() * y_tiles(); i++)
@@ -342,8 +378,8 @@ void Board::newGame() {
     // redistribute unsolved tiles
     while(num_tiles > 0) {
       // get a random tile
-      int r1 = random(num_tiles);
-      int r2 = random(num_tiles);
+      int r1 = random.getLong(num_tiles);
+      int r2 = random.getLong(num_tiles);
       int tile = tiles[r1];
       int apos = pos[r2];
       
@@ -383,7 +419,7 @@ void Board::updateField(int x, int y) {
 	  pm_tile[0]->width(),
 	  pm_tile[0]->height());
   if(isUpdatesEnabled())
-    repaint(r, TRUE);
+    repaint(r, true);
 }
 
 int MIN(int a, int b) {
@@ -423,36 +459,40 @@ QPixmap *Board::lighten(QPixmap *src) {
   return pm;
 }
 
-void Board::paintEvent(QPaintEvent *e) {
+void Board::paintEvent(QPaintEvent *e) {  
   QPainter p;
   p.begin(this);
-  for(int i = 0; i < x_tiles(); i++)
-    for(int j = 0; j < y_tiles(); j++) {
-      if(getField(i, j) == EMPTY)
-	continue;
 
-      int xpos = XBORDER + i * pm_tile[1]->width();
-      int ypos = YBORDER + j * pm_tile[1]->height();
-      QRect r(xpos, ypos, pm_tile[1]->width(), pm_tile[1]->height());
-      if(e->rect().intersects(r)) {
-	// check if it is a marked piece
-	if(i == mark_x && j == mark_y) {
+  if(paused) {
+    if(!pausedIcon)
+      pausedIcon = new QPixmap(KGlobal::dirs()->findResource("appdata", "paused.xpm"));
+
+    p.drawPixmap((width()-pausedIcon->width())/2, 
+		 (height()-pausedIcon->height())/2, 
+		 *pausedIcon);
+  } else {
+    for(int i = 0; i < x_tiles(); i++)
+      for(int j = 0; j < y_tiles(); j++) {
+	if(getField(i, j) == EMPTY)
+	  continue;
+
+	int xpos = XBORDER + i * pm_tile[1]->width();
+	int ypos = YBORDER + j * pm_tile[1]->height();
+	QRect r(xpos, ypos, pm_tile[1]->width(), pm_tile[1]->height());
+	if(e->rect().intersects(r)) {
+	  // check if it is a marked piece
+	  if(i == mark_x && j == mark_y) {
 	    QPixmap *lpm = lighten(pm_tile[getField(i, j)-1]);
 	    p.drawPixmap(xpos, ypos, *lpm);
 	    delete lpm;
-	} else
-	  p.drawPixmap(xpos, ypos, *pm_tile[getField(i, j)-1]);
+	  } else
+	    p.drawPixmap(xpos, ypos, *pm_tile[getField(i, j)-1]);
+	}
       }
-    }
+  }
   p.end();
 }
 
-// returns a random number < max
-int Board::random(int max) {
-  //return (int)((float)(max) * rand() / (RAND_MAX+1.0));
-  return ::random() % max; // don't depend on RAND_MAX...
-}
-
 void Board::marked(int x, int y) {
   // make sure that the last arrow is correctly undrawn
   undrawArrow();
@@ -489,6 +529,8 @@ void Board::marked(int x, int y) {
       drawArrow(mark_x, mark_y, x, y);
       setField(mark_x, mark_y, EMPTY);
       setField(x, y, EMPTY);
+      grav_col_1 = x;
+      grav_col_2 = mark_x;
       mark_x = -1;
       mark_y = -1;
 
@@ -497,7 +539,7 @@ void Board::marked(int x, int y) {
 
       // game is over?      
       if(!getHint_I(dummyx,dummyx,dummyx,dummyx,dummyh)) {
-	time_for_game = (int)time((time_t)NULL) - starttime;
+	time_for_game = (int)difftime( time((time_t)0), starttime);
 	emit endOfGame();
       }
       
@@ -514,27 +556,27 @@ bool Board::canMakePath(int x1, int y1, int x2, int y2) {
   if(x1 == x2) {
     for(i = MIN(y1, y2)+1; i < MAX(y1, y2); i++) 
       if(getField(x1, i) != EMPTY)
-	return FALSE;
+	return false;
   
-    return TRUE;
+    return true;
   } 
 
   if(y1 == y2) {
     for(i = MIN(x1, x2)+1; i < MAX(x1, x2); i++)
       if(getField(i, y1) != EMPTY)
-	return FALSE;
+	return false;
 
-    return TRUE;
+    return true;
   }
 
-  return FALSE;
+  return false;
 }
 
 bool Board::findPath(int x1, int y1, int x2, int y2) {
  clearHistory();
 
   if(findSimplePath(x1, y1, x2, y2))
-     return TRUE;
+     return true;
   else {
     // find 3-way path
     int dx[4] = {1, 0, -1, 0};
@@ -554,7 +596,7 @@ bool Board::findPath(int x1, int y1, int x2, int y2) {
 	  // insert history point
 	  history[0].x = x1;
 	  history[0].y = y1;
-	  return TRUE;	 
+	  return true;	 
 	}
 
 	newx += dx[i];
@@ -563,14 +605,14 @@ bool Board::findPath(int x1, int y1, int x2, int y2) {
     }
 
     clearHistory();
-    return FALSE;
+    return false;
   }
 
-  return FALSE;
+  return false;
 }
 
 bool Board::findSimplePath(int x1, int y1, int x2, int y2) {
-  bool r = FALSE;
+  bool r = false;
 
   // find direct line
   if(canMakePath(x1, y1, x2, y2)) {
@@ -578,7 +620,7 @@ bool Board::findSimplePath(int x1, int y1, int x2, int y2) {
     history[0].y = y1;
     history[1].x = x2;
     history[1].y = y2;
-    r = TRUE;
+    r = true;
   } else {
     if(!(x1 == x2 || y1 == y2)) // requires complex path
       if(getField(x2, y1) == EMPTY &&
@@ -589,7 +631,7 @@ bool Board::findSimplePath(int x1, int y1, int x2, int y2) {
 	history[1].y = y1;
 	history[2].x = x2;
 	history[2].y = y2;
-	r = TRUE;
+	r = true;
       } else if(getField(x1, y2) == EMPTY &&
 		canMakePath(x1, y1, x1, y2) && canMakePath(x1, y2, x2, y2)) {
 	history[0].x = x1;
@@ -598,7 +640,7 @@ bool Board::findSimplePath(int x1, int y1, int x2, int y2) {
 	history[1].y = y2;
 	history[2].x = x2;
 	history[2].y = y2;
-	r = TRUE;
+	r = true;
       }
   }
   
@@ -645,6 +687,13 @@ void Board::undrawArrow() {
   if(trying)
     return;
 
+  if(grav_col_1 != -1 || grav_col_2 != -1) {
+    gravity(grav_col_1, true);
+    gravity(grav_col_2, true);
+    grav_col_1 = -1;
+    grav_col_2 = -1;
+  }
+
   // is already undrawn?
   if(history[0].x == -2)
     return;
@@ -718,6 +767,41 @@ void Board::undo() {
   if(canUndo()) {
     undrawArrow();
     Move *m = _undo.take(_undo.count() - 1);
+    if(gravityFlag()) {
+      int y;
+      int delta = 1;
+      
+      if(m->x1 == m->x2) {
+	delta++;
+
+	/* damned, I hate this. This undo/redo stuff is really complicated
+	 * when used with gravity. This avoids a bug when both tiles reside
+	 * in the same row, but not adjascent y positions. In that case, the
+	 * order of undo is important
+	 */
+	if(m->y1>m->y2) {
+	  int t = m->x1;
+	  m->x1 = m->x2;
+	  m->x2 = t;
+	  t = m->y1;
+	  m->y1 = m->y2;
+	  m->y2 = t;
+	}
+      }
+      
+      for(y = 0; y <= m->y1-1; y++) {
+	setField(m->x1, y, getField(m->x1, y+delta));
+	updateField(m->x1, y);
+      }
+     
+      if(m->x1 != m->x2) {
+	for(y = 0; y < m->y2; y++) {
+	  setField(m->x2, y, getField(m->x2, y+1));
+	  updateField(m->x2, y);
+	}
+      }
+    }
+
     setField(m->x1, m->y1, m->tile);
     setField(m->x2, m->y2, m->tile);
     updateField(m->x1, m->y1);
@@ -735,6 +819,8 @@ void Board::redo() {
     setField(m->x2, m->y2, EMPTY);
     updateField(m->x1, m->y1);
     updateField(m->x2, m->y2);
+    gravity(m->x1, true);
+    gravity(m->x2, true);
     _undo.append(m);
     emit changed();
   }
@@ -769,17 +855,30 @@ void Board::getHint() {
   }
 }
 
+
 #ifdef DEBUGGING
+void Board::makeHintMove() {
+  int x1, y1, x2, y2;
+  History h[4];
+
+  if(getHint_I(x1, y1, x2, y2, h)) {
+    mark_x = -1;
+    mark_y = -1;
+    marked(x1, y1);
+    marked(x2, y2);
+  }
+}
+
 void Board::finish() {
   int x1, y1, x2, y2;
   History h[4];
-  bool ready=FALSE;
+  bool ready=false;
 
   while(!ready && getHint_I(x1, y1, x2, y2, h)) {
     mark_x = -1;
     mark_y = -1;
     if(tilesLeft() == 2)
-      ready = TRUE;
+      ready = true;
     marked(x1, y1);
     marked(x2, y2);
     kapp->processEvents();
@@ -824,7 +923,7 @@ bool Board::getHint_I(int &x1, int &y1, int &x2, int &y2, History h[4]) {
 		  y2 = yy;
 		  for(int i = 0; i < 4; i++)
 		    history[i] = old[i];
-		  return TRUE;
+		  return true;
 		}
 	
 	clearHistory();
@@ -834,7 +933,7 @@ bool Board::getHint_I(int &x1, int &y1, int &x2, int &y2, History h[4]) {
   for(int i = 0; i < 4; i++)
     history[i] = old[i];
 
-  return FALSE;
+  return false;
 }
 
 void Board::setShuffle(int newvalue) {
@@ -857,14 +956,17 @@ int Board::tilesLeft() {
 }
 
 int Board::getCurrentTime() {
-  return (int)(time((time_t *)0) - starttime);
+  return (int)difftime(time((time_t *)0),starttime);
 }
 
 int Board::getTimeForGame() {
   if(tilesLeft() == 0) 
     return time_for_game;
   else
-    return (int)(time((time_t *)0) - starttime);
+    if(paused)
+      return (int)difftime(pause_start, starttime);
+    else
+      return (int)difftime(time((time_t *)0), starttime);
 }
 
 bool Board::solvable(bool norestore) {
@@ -873,20 +975,24 @@ bool Board::solvable(bool norestore) {
   int *oldfield = 0;
  
   if(!norestore) {
-    oldfield = (int *)malloc(x_tiles() * y_tiles() * sizeof(int));
+    oldfield = new int [x_tiles() * y_tiles()];
     memcpy(oldfield, field, x_tiles() * y_tiles() * sizeof(int));
   }
 
   while(getHint_I(x1, y1, x2, y2, h)) {
     setField(x1, y1, EMPTY);
     setField(x2, y2, EMPTY);
+//      if(gravityFlag()) {
+//        gravity(x1, false);
+//        gravity(x2, false);
+//      }
   }
   
   int left = tilesLeft();
 
   if(!norestore) {
     memcpy(field, oldfield, x_tiles() * y_tiles() * sizeof(int));
-    free(oldfield);  
+    delete [] oldfield;  
   }
 
   return (bool)(left == 0);
@@ -900,4 +1006,23 @@ void Board::setSolvableFlag(bool value) {
   _solvable_flag = value;
 }
 
+bool Board::gravityFlag() {
+  return gravity_flag;
+}
+
+void Board::setGravityFlag(bool b) {
+  gravity_flag = b;
+}
+
+bool Board::pause() {
+  paused = !paused;
+  if(paused)
+    pause_start = time((time_t *)0);
+  else
+    starttime += (time_t) difftime( time((time_t *)0), pause_start);
+  update();
+
+  return paused;
+}
+
 #include "board.moc"
