@@ -41,6 +41,30 @@ const char *HS_LEVEL     = "level";
 #define HSScore(i) HSString(HS_SCORE, i)
 #define HSLevel(i) HSString(HS_LEVEL, i)
 
+uint HighScores::nbEntries()
+{
+	KConfig *conf = kapp->config();
+	conf->setGroup(HS_GRP);
+
+	for (uint i=0; i<NB_HS; i++)
+		if ( conf->readUnsignedNumEntry(HSScore(i), 0)==0 ) return i;
+	return NB_HS;
+}
+
+int HighScores::betterEntry(const GameOverItem *goi)
+{
+	KConfig *conf = kapp->config();
+	conf->setGroup(HS_GRP);
+
+	// (we assume they are ordered and valid ...)
+	uint ne = nbEntries();
+	for (uint i=0; i<ne; i++) {
+		uint score = conf->readUnsignedNumEntry(HSScore(i), 0);
+		if ( goi && score<goi->score ) return i;
+	}
+	return ne<NB_HS ? (int)ne : -1;
+}
+
 HighScores::HighScores(const GameOverItem *goi, QWidget *parent,
 					   const char *name)
 : DialogBase(i18n("Hall of Fame"), Close, Close, parent, name), qle(0)
@@ -48,34 +72,22 @@ HighScores::HighScores(const GameOverItem *goi, QWidget *parent,
 	KConfig *conf = kapp->config();
 	conf->setGroup(HS_GRP);
 
-	// find the number of score entries and the position of
-	// the new score (if set)
-	// (we assume they are ordered and valid ...)
-	k = NB_HS;
-	uint s, i;
-	for (i=0; i<NB_HS; i++) {
-		s = conf->readNumEntry(HSScore(i), 0);
-		if ( s==0 ) { // end of entries
-				if ( k==NB_HS ) k = i;
-			break;
-		}
-		if ( goi && s<goi->score && k==NB_HS ) k = i;
-	}
+	bEntry = betterEntry(goi);
+	uint nEntry = nbEntries();
 	
 	if (goi) {
-		if ( k==NB_HS ) return; // not a better score
+		ASSERT( bEntry!=-1 );
 
 		// insert the new score
-		i++;
-		for (uint j=i-1; j>k; j--) {
+		if ( nEntry<NB_HS ) nEntry++;
+		for (uint j=nEntry-1; j>(uint)bEntry; j--) {
 			conf->writeEntry(HSScore(j), conf->readEntry(HSScore(j-1)));
             conf->writeEntry(HSName(j), conf->readEntry(HSName(j-1)));
             conf->writeEntry(HSLevel(j), conf->readEntry(HSLevel(j-1)));
 		}
-		
-		conf->writeEntry(HSScore(k), (uint)goi->score);
-		conf->writeEntry(HSName(k), i18n("Anonymous")); // default name
-		conf->writeEntry(HSLevel(k), (uint)goi->level);
+		conf->writeEntry(HSScore(bEntry), (uint)goi->score);
+		conf->writeEntry(HSName(bEntry), i18n("Anonymous")); // default name
+		conf->writeEntry(HSLevel(bEntry), (uint)goi->level);
 	}
 
 /* layout */
@@ -84,13 +96,13 @@ HighScores::HighScores(const GameOverItem *goi, QWidget *parent,
 	QFont f( font() );
 	f.setBold(TRUE);
 
-	if ( i==0 ) {
+	if ( nEntry==0 ) {
 		lab = new QLabel(i18n("no score entry"), plainPage());
 		lab->setMinimumSize( lab->sizeHint() );
 		lab->setFont(f);
 		top->addWidget(lab);
 	} else {
-		gl = new QGridLayout(top, 4, i+1, 2*spacingHint());
+		gl = new QGridLayout(top, 4, nEntry+1, 2*spacingHint());
 		
 		lab = new QLabel(i18n("Rank"), plainPage());
 		lab->setMinimumSize( lab->sizeHint() );
@@ -111,7 +123,7 @@ HighScores::HighScores(const GameOverItem *goi, QWidget *parent,
 		gl->addWidget(lab, 0, 3);
 	}
 	
-	for (uint j=0; j<i; j++) {
+	for (uint j=0; j<nEntry; j++) {
 		/* rank */
 		QString str;
 		str.sprintf("%i", j+1);
@@ -120,7 +132,7 @@ HighScores::HighScores(const GameOverItem *goi, QWidget *parent,
 		gl->addWidget(lab, j+1, 0);
 
 		/* name */
-		if ( !goi || (k!=j) ) {
+		if ( !goi || ((uint)bEntry!=j) ) {
 			QString name = conf->readEntry(HSName(j));
 			lab = new QLabel(plainPage());
 			lab->setFont(f);
@@ -162,9 +174,9 @@ void HighScores::writeName()
 	KConfig *conf = kapp->config();
 	conf->setGroup(HS_GRP);
 	QString str = qle->text();
-	if ( str.length() ) conf->writeEntry(HSName(k), str);
+	if ( str.length() ) conf->writeEntry(HSName(bEntry), str);
 	conf->sync();
-	str = conf->readEntry(HSName(k));
+	str = conf->readEntry(HSName(bEntry));
 	qle->setText(str);
 	enableButton(Close, TRUE);
 }
