@@ -30,13 +30,17 @@ class HighScores : public DialogBase
  public:
     HighScores(const GameOverItem *, QWidget *parent, const char *name = 0);
 
+	static int betterEntry(const GameOverItem *);
+
  private slots:
     void reject();
 	void writeName();
 	
  private:
-	uint         k;
-	QLineEdit   *qle;
+	int        bEntry;
+	QLineEdit *qle;
+
+	static uint nbEntries();
 };
 
 //-----------------------------------------------------------------------------
