@@ -41,9 +41,7 @@ GenericTetris::~GenericTetris()
 {
 	delete _currentPiece;
 	delete _nextPiece;
-	for (uint i=0; i<_matrix.width(); i++)
-		for (uint j=0; j<_matrix.height(); j++)
-			delete _matrix(i, j);
+	clear();
 }
 
 void GenericTetris::setBlockInfo(BlockInfo *main, BlockInfo *next)
@@ -51,6 +49,7 @@ void GenericTetris::setBlockInfo(BlockInfo *main, BlockInfo *next)
 	ASSERT( _graphic );
 	_nextPiece->setBlockInfo(next);
 	_currentPiece->setBlockInfo(main);
+	clear(); // on destruction : prevent segfault
 }
 
 void GenericTetris::init(int seed)
