@@ -2,19 +2,17 @@
 #include <qpixmap.h>
 #include <qlist.h>
 
-#include <stdlib.h>
-
 #include "basket.h"
 #include "board.h"
 #include "pixServer.h"
 #include "qtimer.h"
 
-Kaffee::Kaffee(int pos)
+Kaffee::Kaffee(int pos, int r1, int r2)
 {
     p = pos;
     t = Red;
-    int r = random() % 40000;
-    QTimer::singleShot( r, this, SLOT(golden()) );
+    r = r2;
+    QTimer::singleShot( r1, this, SLOT(golden()) );
     dirty = TRUE;
 }
 
@@ -22,7 +20,6 @@ void Kaffee::golden()
 {
     dirty = TRUE;
     t = (t == Red ? Golden : Red);
-    int r = random() % 40000;
     QTimer::singleShot( r, this, SLOT(golden()) );
 }
 
@@ -46,9 +43,9 @@ void Basket::newApples()
     int i = 0;
 
     while(i < 10) {
-	x =  random() % board->size();
+	x =  random.getLong(board->size());
 	if ((unsigned)x < board->size() && board->isEmpty(x) && x > BoardWidth+4) {
-	    Kaffee *g = new Kaffee(x);
+	    Kaffee *g = new Kaffee(x, random.getLong(40000), random.getLong(40000));
 	    board->set(x, Apple);
 	    list->append(g);
 	    i++;
