@@ -7,6 +7,8 @@
 #include <qwidget.h>
 #include <qlist.h>
 
+#include <krandomsequence.h>
+
 #include "board.h"
 #include "pixServer.h"
 
@@ -16,7 +18,7 @@ class Kaffee : public QObject
 {
     Q_OBJECT
 public:
-    Kaffee(int pos);
+    Kaffee(int pos, int r1, int r2);
     int position() { return p;}
     Fruits type() { return t;}
     bool dirty;
@@ -24,6 +26,7 @@ private slots:
     void golden();
 private:
     int p;
+    int r;
     Fruits t;
 };
 
@@ -42,6 +45,7 @@ private:
     Board   *board;
     PixServer *pixServer;
     QList<Kaffee> *list;
+    KRandomSequence random;
 };
 
 #endif // BASKET_H
