@@ -1,12 +1,12 @@
-#include <qdatetm.h>
+#include <qdatetime.h>
 #include <stdlib.h>
 #include <qtimer.h>
 #include <qlist.h>
 #include <qcolor.h>
 #include <qlabel.h>
 #include <qkeycode.h>
-#include <qmsgbox.h>
-#include <qbitarry.h>
+#include <qmessagebox.h>
+#include <qbitarray.h>
 
 #include <kapp.h>
 #include <kaccel.h>
@@ -18,6 +18,8 @@
 #include "ball.h"
 #include "snake.h"
 #include "pixServer.h"
+#include <klocale.h>
+#include <kconfig.h>
 
 
 QBitArray gameState(5);
@@ -27,10 +29,9 @@ int speed[4] = { 85, 75, 55, 40 };
 Rattler::Rattler( QWidget *parent, const char *name )
     : QWidget( parent, name )
 {
-
     setFocusPolicy(QWidget::StrongFocus);
 
-    KConfig *conf = kapp->getConfig();
+    KConfig *conf = kapp->config();
     numBalls = conf->readNumEntry("Balls", 1);
     numSnakes = conf->readNumEntry("ComputerSnakes", 1);
     skill = conf->readNumEntry("Skill", 1);
@@ -43,7 +44,7 @@ Rattler::Rattler( QWidget *parent, const char *name )
     level = new Level(board);
 
     pix = new PixServer(board, this);
-    setFixedSize(pix->levelPix().size());
+    // setFixedSize(pix->levelPix().size());
 
     basket = new Basket(board, pix);
     samy = new SamySnake(board, pix);
@@ -59,14 +60,12 @@ Rattler::Rattler( QWidget *parent, const char *name )
     connect( samy, SIGNAL(goingOut()), this, SLOT(speedUp()));
     connect( basket, SIGNAL(openGate()), this, SLOT(openGate()));
 
-    QTime midnight( 0, 0, 0 );
-    srandom( midnight.secsTo(QTime::currentTime()) );
-
     gameState.fill(FALSE);
     gameState.setBit(Demo);
 
     timerCount = 0;
     QTimer::singleShot( 2000, this, SLOT(demo()) );
+
 }
 
 void Rattler::paintEvent( QPaintEvent *e)
@@ -74,27 +73,16 @@ void Rattler::paintEvent( QPaintEvent *e)
     QRect rect = e->rect();
     bool dirty = FALSE;
 
-    if (!rect.isEmpty()) {
-	dirty = TRUE;
-	QPixmap levelPix = pix->levelPix();
-	bitBlt(this, rect.x(), rect.y(),
-	       &levelPix, rect.x(), rect.y(), rect.width(), rect.height());
-    }
+    if (rect.isEmpty())
+      return;
+    dirty = TRUE;
+    QPixmap levelPix = pix->levelPix();
+    bitBlt(this, rect.x(), rect.y(),
+              &levelPix, rect.x(), rect.y(), rect.width(), rect.height());
+    //most Below here was moved to timerEvent where it belongs...
+  
 
-    if (!gameState.testBit(Init) && !gameState.testBit(Over)) {
 
-	basket->repaint( dirty );
-
-	if(!gameState.testBit(Demo))samy->repaint( dirty );
-
-	for (CompuSnake *as = computerSnakes->first(); as != 0;
-	     as = computerSnakes->next())
-	    if (as) as->repaint( dirty );
-
-	for (Ball *b = balls->first(); b != 0;
-	     b = balls->next())
-	    if (b) b->repaint();
-    }
 }
 
 void Rattler::timerEvent( QTimerEvent * )
@@ -117,7 +105,20 @@ void Rattler::timerEvent( QTimerEvent * )
     if(!gameState.testBit(Demo))
 	state = samy->nextMove(direction);
 
-    repaint(0,0,0,0, FALSE);
+    //    repaint(0,0,1,1, FALSE);
+    // Below here was moved from paintEvent, which was causing unnecessary paints....
+
+	if(!gameState.testBit(Demo))
+	  samy->repaint( false );
+        basket->repaint( false);
+	for (CompuSnake *as = computerSnakes->first(); as != 0;
+	     as = computerSnakes->next())
+	    if (as) as->repaint(false);
+
+	for (Ball *b = balls->first(); b != 0;
+	     b = balls->next())
+	    if (b) b->repaint();
+
 
     if (state == ko)
 	newTry();
@@ -127,23 +128,23 @@ void Rattler::timerEvent( QTimerEvent * )
 
 void Rattler::initKeys()
 {
-    KConfig *conf = kapp->getConfig();
+    KConfig *conf = kapp->config();
 
     QString up("Up");
-    up = conf->readEntry("upKey", (const char*) up);
-    UpKey    = stringToKey(up);
+    up = conf->readEntry("upKey", up);
+    UpKey    = KAccel::stringToKey(up);
 
     QString down("Down");
-    down = conf->readEntry("downKey", (const char*) down);
-    DownKey  = stringToKey(down);
+    down = conf->readEntry("downKey", down);
+    DownKey  = KAccel::stringToKey(down);
 
     QString left("Left");
-    left = conf->readEntry("leftKey", (const char*) left);
-    LeftKey  = stringToKey(left);
+    left = conf->readEntry("leftKey", left);
+    LeftKey  = KAccel::stringToKey(left);
 
     QString right("Right");
-    right = conf->readEntry("rightKey", (const char*) right);
-    RightKey = stringToKey(right);
+    right = conf->readEntry("rightKey", right);
+    RightKey = KAccel::stringToKey(right);
 
 }
 
@@ -275,7 +276,7 @@ void Rattler::pause()
 
 	label = new QLabel(this);
 	label->setFont( QFont( "Times", 16, QFont::Bold ) );
-	label->setText(klocale->translate("Game Paused\n Press F3 to resume\n"));
+	label->setText(i18n("Game Paused\n Press F3 to resume\n"));
 	label->setAlignment( AlignCenter );
 	label->setFrameStyle( QFrame::Panel | QFrame::Raised );
 	label->setGeometry(182, 206, 198, 80);
@@ -304,7 +305,7 @@ void Rattler::restartDemo()
     if (!gameState.testBit(Demo))
 	return;
 
-    int r = 50000+ (random() % 30000);
+    int r = 50000+ (kapp->random() % 30000);
     QTimer::singleShot( r, this, SLOT(restartDemo()) );
 
     stop();
@@ -347,11 +348,11 @@ void Rattler::restart()
 
     if (gameState.testBit(Paused) || gameState.testBit(Playing)) {
 
-	switch( QMessageBox::information( this, klocale->translate("Snake Race"),
-					  klocale->translate("A game is already started\n"
+	switch( QMessageBox::information( this, i18n("Snake Race"),
+					  i18n("A game is already started\n"
 					  "Start a new one ?\n"),
-					  klocale->translate("&Yes"), klocale->translate("&No"),
-					  0, 1 ) ) {
+					  i18n("&Yes"), i18n("&No"),
+					  QString::null, 1 ) ) {
 	case 0:
 	    if ( gameState.testBit(Paused))
 		emit togglePaused();
@@ -554,6 +555,15 @@ void Rattler::setBalls(int i)
     }
 }
 
+void  Rattler::resizeEvent( QResizeEvent * )
+{
+
+    pix->initPixmaps();
+    pix->initBrickPixmap();
+    pix->initbackPixmaps();
+    pix->initRoomPixmap();
+}
+
 void Rattler::setCompuSnakes(int i)
 {
     CompuSnake *cs;
